@@ -15,7 +15,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library;  If not, see
+ * License along with this library.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  * Author: Eric Blake <eblake@redhat.com>
@@ -357,7 +357,7 @@ virDomainSnapshotAlignDisks(virDomainSnapshotDefPtr def,
         goto cleanup;
     }
 
-    if (!(map = virBitmapAlloc(def->dom->ndisks))) {
+    if (!(map = virBitmapNew(def->dom->ndisks))) {
         virReportOOMError();
         goto cleanup;
     }
