@@ -15,7 +15,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library;  If not, see
+ * License along with this library.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  * Author: Daniel P. Berrange <berrange@redhat.com>
@@ -48,6 +48,7 @@
 #include "count-one-bits.h"
 #include "intprops.h"
 #include "virfile.h"
+#include "virtypedparam.h"
 
 
 #define VIR_FROM_THIS VIR_FROM_NONE
@@ -57,6 +58,7 @@
 # define SYSFS_SYSTEM_PATH "/sys/devices/system"
 # define PROCSTAT_PATH "/proc/stat"
 # define MEMINFO_PATH "/proc/meminfo"
+# define SYSFS_MEMORY_SHARED_PATH "/sys/kernel/mm/ksm"
 
 # define LINUX_NB_CPU_STATS 4
 # define LINUX_NB_MEMORY_STATS_ALL 4
@@ -737,10 +739,10 @@ cleanup:
  * and max cpu is 7. The map file shows 0-4,6-7. This function parses
  * it and returns cpumap.
  */
-static char *
+static virBitmapPtr
 linuxParseCPUmap(int *max_cpuid, const char *path)
 {
-    char *map = NULL;
+    virBitmapPtr map = NULL;
     char *str = NULL;
     int max_id = 0, i;
 
@@ -749,20 +751,16 @@ linuxParseCPUmap(int *max_cpuid, const char *path)
         goto error;
     }
 
-    if (VIR_ALLOC_N(map, VIR_DOMAIN_CPUMASK_LEN) < 0) {
-        virReportOOMError();
-        goto error;
-    }
-    if (virDomainCpuSetParse(str, 0, map,
-                             VIR_DOMAIN_CPUMASK_LEN) < 0) {
+    if (virBitmapParse(str, 0, &map,
+                       VIR_DOMAIN_CPUMASK_LEN) < 0) {
         goto error;
     }
 
-    for (i = 0; i < VIR_DOMAIN_CPUMASK_LEN; i++) {
-        if (map[i]) {
-            max_id = i;
-        }
+    i = -1;
+    while ((i = virBitmapNextSetBit(map, i)) >= 0) {
+        max_id = i;
     }
+
     *max_cpuid = max_id;
 
     VIR_FREE(str);
@@ -770,7 +768,7 @@ linuxParseCPUmap(int *max_cpuid, const char *path)
 
 error:
     VIR_FREE(str);
-    VIR_FREE(map);
+    virBitmapFree(map);
     return NULL;
 }
 #endif
@@ -909,14 +907,14 @@ int nodeGetMemoryStats(virConnectPtr conn ATTRIBUTE_UNUSED,
 #endif
 }
 
-char *
+virBitmapPtr
 nodeGetCPUmap(virConnectPtr conn ATTRIBUTE_UNUSED,
               int *max_id ATTRIBUTE_UNUSED,
               const char *mapname ATTRIBUTE_UNUSED)
 {
 #ifdef __linux__
     char *path;
-    char *cpumap;
+    virBitmapPtr cpumap;
 
     if (virAsprintf(&path, SYSFS_SYSTEM_PATH "/cpu/%s", mapname) < 0) {
         virReportOOMError();
@@ -933,6 +931,257 @@ nodeGetCPUmap(virConnectPtr conn ATTRIBUTE_UNUSED,
 #endif
 }
 
+#ifdef __linux__
+static int
+nodeSetMemoryParameterValue(const char *field,
+                            virTypedParameterPtr param)
+{
+    char *path = NULL;
+    char *strval = NULL;
+    int ret = -1;
+    int rc = -1;
+
+    if (virAsprintf(&path, "%s/%s",
+                    SYSFS_MEMORY_SHARED_PATH, field) < 0) {
+        virReportOOMError();
+        ret = -2;
+        goto cleanup;
+    }
+
+    if (virAsprintf(&strval, "%u", param->value.ui) == -1) {
+        virReportOOMError();
+        ret = -2;
+        goto cleanup;
+    }
+
+    if ((rc = virFileWriteStr(path, strval, 0)) < 0) {
+        virReportSystemError(-rc, _("failed to set %s"), field);
+        goto cleanup;
+    }
+
+    ret = 0;
+cleanup:
+    VIR_FREE(path);
+    VIR_FREE(strval);
+    return ret;
+}
+#endif
+
+int
+nodeSetMemoryParameters(virConnectPtr conn ATTRIBUTE_UNUSED,
+                        virTypedParameterPtr params ATTRIBUTE_UNUSED,
+                        int nparams ATTRIBUTE_UNUSED,
+                        unsigned int flags)
+{
+    virCheckFlags(0, -1);
+
+#ifdef __linux__
+    int ret = 0;
+    int i;
+
+    if (virTypedParameterArrayValidate(params, nparams,
+                                       VIR_NODE_MEMORY_SHARED_PAGES_TO_SCAN,
+                                       VIR_TYPED_PARAM_UINT,
+                                       VIR_NODE_MEMORY_SHARED_SLEEP_MILLISECS,
+                                       VIR_TYPED_PARAM_UINT,
+                                       NULL) < 0)
+        return -1;
+
+    for (i = 0; i < nparams; i++) {
+        virTypedParameterPtr param = &params[i];
+
+        if (STREQ(param->field,
+                  VIR_NODE_MEMORY_SHARED_PAGES_TO_SCAN)) {
+            ret = nodeSetMemoryParameterValue("pages_to_scan", param);
+
+            /* Out of memory */
+            if (ret == -2)
+                return -1;
+        } else if (STREQ(param->field,
+                         VIR_NODE_MEMORY_SHARED_SLEEP_MILLISECS)) {
+            ret = nodeSetMemoryParameterValue("sleep_millisecs", param);
+
+            /* Out of memory */
+            if (ret == -2)
+                return -1;
+        }
+    }
+
+    return ret;
+#else
+    virReportError(VIR_ERR_NO_SUPPORT, "%s",
+                   _("node set memory parameters not implemented"
+                     " on this platform"));
+    return -1;
+#endif
+}
+
+#ifdef __linux__
+static int
+nodeGetMemoryParameterValue(const char *field,
+                            void *value)
+{
+    char *path = NULL;
+    char *buf = NULL;
+    char *tmp = NULL;
+    int ret = -1;
+    int rc = -1;
+
+    if (virAsprintf(&path, "%s/%s",
+                    SYSFS_MEMORY_SHARED_PATH, field) < 0) {
+        virReportOOMError();
+        goto cleanup;
+    }
+
+    if (virFileReadAll(path, 1024, &buf) < 0)
+        goto cleanup;
+
+    if ((tmp = strchr(buf, '\n')))
+        *tmp = '\0';
+
+    if (STREQ(field, "pages_to_scan") ||
+        STREQ(field, "sleep_millisecs"))
+        rc = virStrToLong_ui(buf, NULL, 10, (unsigned int *)value);
+    else if (STREQ(field, "pages_shared")    ||
+             STREQ(field, "pages_sharing")   ||
+             STREQ(field, "pages_unshared")  ||
+             STREQ(field, "pages_volatile")  ||
+             STREQ(field, "full_scans"))
+        rc = virStrToLong_ull(buf, NULL, 10, (unsigned long long *)value);
+
+    if (rc < 0) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("failed to parse %s"), field);
+        goto cleanup;
+    }
+
+    ret = 0;
+cleanup:
+    VIR_FREE(path);
+    VIR_FREE(buf);
+    return ret;
+}
+#endif
+
+#define NODE_MEMORY_PARAMETERS_NUM 7
+int
+nodeGetMemoryParameters(virConnectPtr conn ATTRIBUTE_UNUSED,
+                        virTypedParameterPtr params ATTRIBUTE_UNUSED,
+                        int *nparams ATTRIBUTE_UNUSED,
+                        unsigned int flags)
+{
+    virCheckFlags(VIR_TYPED_PARAM_STRING_OKAY, -1);
+
+#ifdef __linux__
+    unsigned int pages_to_scan;
+    unsigned int sleep_millisecs;
+    unsigned long long pages_shared;
+    unsigned long long pages_sharing;
+    unsigned long long pages_unshared;
+    unsigned long long pages_volatile;
+    unsigned long long full_scans = 0;
+    int i;
+
+    if ((*nparams) == 0) {
+        *nparams = NODE_MEMORY_PARAMETERS_NUM;
+        return 0;
+    }
+
+    for (i = 0; i < *nparams && i < NODE_MEMORY_PARAMETERS_NUM; i++) {
+        virTypedParameterPtr param = &params[i];
+
+        switch(i) {
+        case 0:
+            if (nodeGetMemoryParameterValue("pages_to_scan",
+                                            &pages_to_scan) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_PAGES_TO_SCAN,
+                                        VIR_TYPED_PARAM_UINT, pages_to_scan) < 0)
+                return -1;
+
+            break;
+
+        case 1:
+            if (nodeGetMemoryParameterValue("sleep_millisecs",
+                                            &sleep_millisecs) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_SLEEP_MILLISECS,
+                                        VIR_TYPED_PARAM_UINT, sleep_millisecs) < 0)
+                return -1;
+
+            break;
+
+        case 2:
+            if (nodeGetMemoryParameterValue("pages_shared",
+                                            &pages_shared) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_PAGES_SHARED,
+                                        VIR_TYPED_PARAM_ULLONG, pages_shared) < 0)
+                return -1;
+
+            break;
+
+        case 3:
+            if (nodeGetMemoryParameterValue("pages_sharing",
+                                            &pages_sharing) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_PAGES_SHARING,
+                                        VIR_TYPED_PARAM_ULLONG, pages_sharing) < 0)
+                return -1;
+
+            break;
+
+        case 4:
+            if (nodeGetMemoryParameterValue("pages_unshared",
+                                            &pages_unshared) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_PAGES_UNSHARED,
+                                        VIR_TYPED_PARAM_ULLONG, pages_unshared) < 0)
+                return -1;
+
+            break;
+
+        case 5:
+            if (nodeGetMemoryParameterValue("pages_volatile",
+                                            &pages_volatile) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_PAGES_VOLATILE,
+                                        VIR_TYPED_PARAM_ULLONG, pages_volatile) < 0)
+                return -1;
+
+            break;
+
+        case 6:
+            if (nodeGetMemoryParameterValue("full_scans",
+                                            &full_scans) < 0)
+                return -1;
+
+            if (virTypedParameterAssign(param, VIR_NODE_MEMORY_SHARED_FULL_SCANS,
+                                        VIR_TYPED_PARAM_ULLONG, full_scans) < 0)
+                return -1;
+
+            break;
+
+        default:
+            break;
+        }
+    }
+
+    return 0;
+#else
+    virReportError(VIR_ERR_NO_SUPPORT, "%s",
+                   _("node get memory parameters not implemented"
+                     " on this platform"));
+    return -1;
+#endif
+}
+
 #if HAVE_NUMACTL
 # if LIBNUMA_API_VERSION <= 1
 #  define NUMA_MAX_N_CPUS 4096
