@@ -1,7 +1,7 @@
 /*
  * secret_driver.c: local driver for secret manipulation API
  *
- * Copyright (C) 2009-2011 Red Hat, Inc.
+ * Copyright (C) 2009-2012 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,7 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library;  If not, see
+ * License along with this library.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  * Red Hat Author: Miloslav Trmač <mitr@redhat.com>
@@ -601,7 +601,6 @@ cleanup:
     return -1;
 }
 
-
 static const char *
 secretUsageIDForDef(virSecretDefPtr def)
 {
@@ -620,6 +619,86 @@ secretUsageIDForDef(virSecretDefPtr def)
     }
 }
 
+#define MATCH(FLAG) (flags & (FLAG))
+static int
+secretListAllSecrets(virConnectPtr conn,
+                     virSecretPtr **secrets,
+                     unsigned int flags) {
+    virSecretDriverStatePtr driver = conn->secretPrivateData;
+    virSecretPtr *tmp_secrets = NULL;
+    int nsecrets = 0;
+    int ret_nsecrets = 0;
+    virSecretPtr secret = NULL;
+    virSecretEntryPtr entry = NULL;
+    int i = 0;
+    int ret = -1;
+
+    virCheckFlags(VIR_CONNECT_LIST_SECRETS_FILTERS_ALL, -1);
+
+    secretDriverLock(driver);
+
+    for (entry = driver->secrets; entry != NULL; entry = entry->next)
+        nsecrets++;
+
+    if (secrets) {
+        if (VIR_ALLOC_N(tmp_secrets, nsecrets + 1) < 0) {
+            virReportOOMError();
+            goto cleanup;
+        }
+    }
+
+    for (entry = driver->secrets; entry != NULL; entry = entry->next) {
+        /* filter by whether it's ephemeral */
+        if (MATCH(VIR_CONNECT_LIST_SECRETS_FILTERS_EPHEMERAL) &&
+            !((MATCH(VIR_CONNECT_LIST_SECRETS_EPHEMERAL) &&
+               entry->def->ephemeral) ||
+              (MATCH(VIR_CONNECT_LIST_SECRETS_NO_EPHEMERAL) &&
+               !entry->def->ephemeral)))
+            continue;
+
+        /* filter by whether it's private */
+        if (MATCH(VIR_CONNECT_LIST_SECRETS_FILTERS_PRIVATE) &&
+            !((MATCH(VIR_CONNECT_LIST_SECRETS_PRIVATE) &&
+               entry->def->private) ||
+              (MATCH(VIR_CONNECT_LIST_SECRETS_NO_PRIVATE) &&
+               !entry->def->private)))
+            continue;
+
+        if (secrets) {
+            if (!(secret = virGetSecret(conn,
+                                        entry->def->uuid,
+                                        entry->def->usage_type,
+                                        secretUsageIDForDef(entry->def))))
+                goto cleanup;
+            tmp_secrets[ret_nsecrets] = secret;
+        }
+        ret_nsecrets++;
+    }
+
+    if (tmp_secrets) {
+        /* trim the array to the final size */
+        ignore_value(VIR_REALLOC_N(tmp_secrets, ret_nsecrets + 1));
+        *secrets = tmp_secrets;
+        tmp_secrets = NULL;
+    }
+
+    ret = ret_nsecrets;
+
+ cleanup:
+    secretDriverUnlock(driver);
+    if (tmp_secrets) {
+        for (i = 0; i < ret_nsecrets; i ++) {
+            if (tmp_secrets[i])
+                virSecretFree(tmp_secrets[i]);
+        }
+    }
+    VIR_FREE(tmp_secrets);
+
+    return ret;
+}
+#undef MATCH
+
+
 static virSecretPtr
 secretLookupByUUID(virConnectPtr conn, const unsigned char *uuid)
 {
@@ -911,7 +990,7 @@ secretGetValue(virSecretPtr obj, size_t *value_size, unsigned int flags,
 
     if ((internalFlags & VIR_SECRET_GET_VALUE_INTERNAL_CALL) == 0 &&
         secret->def->private) {
-        virReportError(VIR_ERR_OPERATION_DENIED, "%s",
+        virReportError(VIR_ERR_INVALID_SECRET, "%s",
                        _("secret is private"));
         goto cleanup;
     }
@@ -1072,6 +1151,7 @@ static virSecretDriver secretDriver = {
     .close = secretClose, /* 0.7.1 */
     .numOfSecrets = secretNumOfSecrets, /* 0.7.1 */
     .listSecrets = secretListSecrets, /* 0.7.1 */
+    .listAllSecrets = secretListAllSecrets, /* 0.10.2 */
     .lookupByUUID = secretLookupByUUID, /* 0.7.1 */
     .lookupByUsage = secretLookupByUsage, /* 0.7.1 */
     .defineXML = secretDefineXML, /* 0.7.1 */
