@@ -11,14 +11,14 @@
 
 struct testInfo {
     const char *name;
-    virBitmapPtr flags;
+    qemuCapsPtr flags;
     unsigned int version;
     unsigned int is_kvm;
     unsigned int kvm_version;
 };
 
-static void printMismatchedFlags(virBitmapPtr got,
-                                 virBitmapPtr expect)
+static void printMismatchedFlags(qemuCapsPtr got,
+                                 qemuCapsPtr expect)
 {
     int i;
 
@@ -38,7 +38,7 @@ static int testHelpStrParsing(const void *data)
     char *path = NULL;
     char *help = NULL;
     unsigned int version, is_kvm, kvm_version;
-    virBitmapPtr flags = NULL;
+    qemuCapsPtr flags = NULL;
     int ret = -1;
     char *got = NULL;
     char *expected = NULL;
@@ -75,8 +75,8 @@ static int testHelpStrParsing(const void *data)
             goto cleanup;
     }
 
-    got = virBitmapString(flags);
-    expected = virBitmapString(info->flags);
+    got = qemuCapsFlagsString(flags);
+    expected = qemuCapsFlagsString(info->flags);
     if (!got || !expected)
         goto cleanup;
 
@@ -116,7 +116,7 @@ static int testHelpStrParsing(const void *data)
 cleanup:
     VIR_FREE(path);
     VIR_FREE(help);
-    qemuCapsFree(flags);
+    virObjectUnref(flags);
     VIR_FREE(got);
     VIR_FREE(expected);
     return ret;
@@ -138,7 +138,7 @@ mymain(void)
         if (virtTestRun("QEMU Help String Parsing " name,                   \
                         1, testHelpStrParsing, &info) < 0)                  \
             ret = -1;                                                       \
-        qemuCapsFree(info.flags);                                           \
+        virObjectUnref(info.flags);                                         \
     } while (0)
 
     DO_TEST("qemu-0.9.1", 9001, 0, 0,
@@ -540,7 +540,8 @@ mymain(void)
             QEMU_CAPS_NO_ACPI,
             QEMU_CAPS_VIRTIO_BLK_SCSI,
             QEMU_CAPS_VIRTIO_BLK_SG_IO,
-            QEMU_CAPS_CPU_HOST);
+            QEMU_CAPS_CPU_HOST,
+            QEMU_CAPS_BLOCKIO);
     DO_TEST("qemu-kvm-0.12.1.2-rhel62-beta", 12001, 1, 0,
             QEMU_CAPS_VNC_COLON,
             QEMU_CAPS_NO_REBOOT,
@@ -607,7 +608,8 @@ mymain(void)
             QEMU_CAPS_VIRTIO_BLK_SG_IO,
             QEMU_CAPS_DRIVE_COPY_ON_READ,
             QEMU_CAPS_CPU_HOST,
-            QEMU_CAPS_SCSI_CD);
+            QEMU_CAPS_SCSI_CD,
+            QEMU_CAPS_BLOCKIO);
     DO_TEST("qemu-1.0", 1000000, 0, 0,
             QEMU_CAPS_VNC_COLON,
             QEMU_CAPS_NO_REBOOT,
@@ -679,7 +681,8 @@ mymain(void)
             QEMU_CAPS_SCSI_BLOCK,
             QEMU_CAPS_SCSI_CD,
             QEMU_CAPS_IDE_CD,
-            QEMU_CAPS_SCSI_LSI);
+            QEMU_CAPS_SCSI_LSI,
+            QEMU_CAPS_BLOCKIO);
     DO_TEST("qemu-1.1.0", 1001000, 0, 0,
             QEMU_CAPS_VNC_COLON,
             QEMU_CAPS_NO_REBOOT,
@@ -759,7 +762,92 @@ mymain(void)
             QEMU_CAPS_NEC_USB_XHCI,
             QEMU_CAPS_NETDEV_BRIDGE,
             QEMU_CAPS_SCSI_LSI,
-            QEMU_CAPS_VIRTIO_SCSI_PCI);
+            QEMU_CAPS_VIRTIO_SCSI_PCI,
+            QEMU_CAPS_BLOCKIO);
+    DO_TEST("qemu-1.2.0", 1002000, 0, 0,
+            QEMU_CAPS_VNC_COLON,
+            QEMU_CAPS_NO_REBOOT,
+            QEMU_CAPS_DRIVE,
+            QEMU_CAPS_NAME,
+            QEMU_CAPS_UUID,
+            QEMU_CAPS_MIGRATE_QEMU_TCP,
+            QEMU_CAPS_MIGRATE_QEMU_EXEC,
+            QEMU_CAPS_DRIVE_CACHE_V2,
+            QEMU_CAPS_DRIVE_CACHE_UNSAFE,
+            QEMU_CAPS_DRIVE_FORMAT,
+            QEMU_CAPS_DRIVE_SERIAL,
+            QEMU_CAPS_XEN_DOMID,
+            QEMU_CAPS_DRIVE_READONLY,
+            QEMU_CAPS_VGA,
+            QEMU_CAPS_0_10,
+            QEMU_CAPS_MEM_PATH,
+            QEMU_CAPS_SDL,
+            QEMU_CAPS_MIGRATE_QEMU_UNIX,
+            QEMU_CAPS_CHARDEV,
+            QEMU_CAPS_ENABLE_KVM,
+            QEMU_CAPS_MONITOR_JSON,
+            QEMU_CAPS_BALLOON,
+            QEMU_CAPS_DEVICE,
+            QEMU_CAPS_SMP_TOPOLOGY,
+            QEMU_CAPS_NETDEV,
+            QEMU_CAPS_RTC,
+            QEMU_CAPS_VHOST_NET,
+            QEMU_CAPS_NO_HPET,
+            QEMU_CAPS_PCI_CONFIGFD,
+            QEMU_CAPS_NODEFCONFIG,
+            QEMU_CAPS_BOOT_MENU,
+            QEMU_CAPS_FSDEV,
+            QEMU_CAPS_NAME_PROCESS,
+            QEMU_CAPS_SMBIOS_TYPE,
+            QEMU_CAPS_VGA_QXL,
+            QEMU_CAPS_SPICE,
+            QEMU_CAPS_VGA_NONE,
+            QEMU_CAPS_MIGRATE_QEMU_FD,
+            QEMU_CAPS_BOOTINDEX,
+            QEMU_CAPS_HDA_DUPLEX,
+            QEMU_CAPS_DRIVE_AIO,
+            QEMU_CAPS_PCI_BOOTINDEX,
+            QEMU_CAPS_CCID_EMULATED,
+            QEMU_CAPS_CCID_PASSTHRU,
+            QEMU_CAPS_VIRTIO_TX_ALG,
+            QEMU_CAPS_PCI_MULTIFUNCTION,
+            QEMU_CAPS_VIRTIO_IOEVENTFD,
+            QEMU_CAPS_SGA,
+            QEMU_CAPS_VIRTIO_BLK_EVENT_IDX,
+            QEMU_CAPS_VIRTIO_NET_EVENT_IDX,
+            QEMU_CAPS_DRIVE_CACHE_DIRECTSYNC,
+            QEMU_CAPS_PIIX3_USB_UHCI,
+            QEMU_CAPS_PIIX4_USB_UHCI,
+            QEMU_CAPS_USB_EHCI,
+            QEMU_CAPS_ICH9_USB_EHCI1,
+            QEMU_CAPS_VT82C686B_USB_UHCI,
+            QEMU_CAPS_PCI_OHCI,
+            QEMU_CAPS_USB_HUB,
+            QEMU_CAPS_NO_SHUTDOWN,
+            QEMU_CAPS_PCI_ROMBAR,
+            QEMU_CAPS_ICH9_AHCI,
+            QEMU_CAPS_NO_ACPI,
+            QEMU_CAPS_FSDEV_READONLY,
+            QEMU_CAPS_VIRTIO_BLK_SCSI,
+            QEMU_CAPS_VIRTIO_BLK_SG_IO,
+            QEMU_CAPS_DRIVE_COPY_ON_READ,
+            QEMU_CAPS_CPU_HOST,
+            QEMU_CAPS_FSDEV_WRITEOUT,
+            QEMU_CAPS_DRIVE_IOTUNE,
+            QEMU_CAPS_SCSI_DISK_CHANNEL,
+            QEMU_CAPS_SCSI_BLOCK,
+            QEMU_CAPS_SCSI_CD,
+            QEMU_CAPS_IDE_CD,
+            QEMU_CAPS_NO_USER_CONFIG,
+            QEMU_CAPS_HDA_MICRO,
+            QEMU_CAPS_NEC_USB_XHCI,
+            QEMU_CAPS_NETDEV_BRIDGE,
+            QEMU_CAPS_SCSI_LSI,
+            QEMU_CAPS_VIRTIO_SCSI_PCI,
+            QEMU_CAPS_BLOCKIO,
+            QEMU_CAPS_SCSI_DISK_WWN,
+            QEMU_CAPS_SECCOMP_SANDBOX,
+            QEMU_CAPS_DUMP_GUEST_CORE);
 
     return ret == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
