@@ -14,7 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library;  If not, see
+ * License along with this library.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  *  Daniel Veillard <veillard@redhat.com>
@@ -36,6 +36,7 @@
 #include "memory.h"
 #include "util.h"
 #include "xml.h"
+#include "conf/network_conf.h"
 
 virNetworkPtr
 vshCommandOptNetworkBy(vshControl *ctl, const vshCmd *cmd,
@@ -100,9 +101,6 @@ cmdNetworkAutostart(vshControl *ctl, const vshCmd *cmd)
     const char *name;
     int autostart;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, &name)))
         return false;
 
@@ -148,9 +146,6 @@ cmdNetworkCreate(vshControl *ctl, const vshCmd *cmd)
     bool ret = true;
     char *buffer;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (vshCommandOptString(cmd, "file", &from) <= 0)
         return false;
 
@@ -193,9 +188,6 @@ cmdNetworkDefine(vshControl *ctl, const vshCmd *cmd)
     bool ret = true;
     char *buffer;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (vshCommandOptString(cmd, "file", &from) <= 0)
         return false;
 
@@ -237,9 +229,6 @@ cmdNetworkDestroy(vshControl *ctl, const vshCmd *cmd)
     bool ret = true;
     const char *name;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, &name)))
         return false;
 
@@ -278,9 +267,6 @@ cmdNetworkDumpXML(vshControl *ctl, const vshCmd *cmd)
     unsigned int flags = 0;
     int inactive;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, NULL)))
         return false;
 
@@ -325,9 +311,6 @@ cmdNetworkInfo(vshControl *ctl, const vshCmd *cmd)
     int active = -1;
     char *bridge = NULL;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, NULL)))
         return false;
 
@@ -360,6 +343,223 @@ cmdNetworkInfo(vshControl *ctl, const vshCmd *cmd)
     return true;
 }
 
+static int
+vshNetworkSorter(const void *a, const void *b)
+{
+    virNetworkPtr *na = (virNetworkPtr *) a;
+    virNetworkPtr *nb = (virNetworkPtr *) b;
+
+    if (*na && !*nb)
+        return -1;
+
+    if (!*na)
+        return *nb != NULL;
+
+    return vshStrcasecmp(virNetworkGetName(*na),
+                      virNetworkGetName(*nb));
+}
+
+struct vshNetworkList {
+    virNetworkPtr *nets;
+    size_t nnets;
+};
+typedef struct vshNetworkList *vshNetworkListPtr;
+
+static void
+vshNetworkListFree(vshNetworkListPtr list)
+{
+    int i;
+
+    if (list && list->nnets) {
+        for (i = 0; i < list->nnets; i++) {
+            if (list->nets[i])
+                virNetworkFree(list->nets[i]);
+        }
+        VIR_FREE(list->nets);
+    }
+    VIR_FREE(list);
+}
+
+static vshNetworkListPtr
+vshNetworkListCollect(vshControl *ctl,
+                      unsigned int flags)
+{
+    vshNetworkListPtr list = vshMalloc(ctl, sizeof(*list));
+    int i;
+    int ret;
+    char **names = NULL;
+    virNetworkPtr net;
+    bool success = false;
+    size_t deleted = 0;
+    int persistent;
+    int autostart;
+    int nActiveNets = 0;
+    int nInactiveNets = 0;
+    int nAllNets = 0;
+
+    /* try the list with flags support (0.10.2 and later) */
+    if ((ret = virConnectListAllNetworks(ctl->conn,
+                                         &list->nets,
+                                         flags)) >= 0) {
+        list->nnets = ret;
+        goto finished;
+    }
+
+    /* check if the command is actually supported */
+    if (last_error && last_error->code == VIR_ERR_NO_SUPPORT)
+        goto fallback;
+
+    if (last_error && last_error->code ==  VIR_ERR_INVALID_ARG) {
+        /* try the new API again but mask non-guaranteed flags */
+        unsigned int newflags = flags & (VIR_CONNECT_LIST_NETWORKS_ACTIVE |
+                                         VIR_CONNECT_LIST_NETWORKS_INACTIVE);
+
+        vshResetLibvirtError();
+        if ((ret = virConnectListAllNetworks(ctl->conn, &list->nets,
+                                             newflags)) >= 0) {
+            list->nnets = ret;
+            goto filter;
+        }
+    }
+
+    /* there was an error during the first or second call */
+    vshError(ctl, "%s", _("Failed to list networks"));
+    goto cleanup;
+
+
+fallback:
+    /* fall back to old method (0.10.1 and older) */
+    vshResetLibvirtError();
+
+    /* Get the number of active networks */
+    if (!VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_ACTIVE) ||
+        VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_ACTIVE)) {
+        if ((nActiveNets = virConnectNumOfNetworks(ctl->conn)) < 0) {
+            vshError(ctl, "%s", _("Failed to get the number of active networks"));
+            goto cleanup;
+        }
+    }
+
+    /* Get the number of inactive networks */
+    if (!VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_ACTIVE) ||
+        VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_INACTIVE)) {
+        if ((nInactiveNets = virConnectNumOfDefinedNetworks(ctl->conn)) < 0) {
+            vshError(ctl, "%s", _("Failed to get the number of inactive networks"));
+            goto cleanup;
+        }
+    }
+
+    nAllNets = nActiveNets + nInactiveNets;
+
+    if (nAllNets == 0)
+         return list;
+
+    names = vshMalloc(ctl, sizeof(char *) * nAllNets);
+
+    /* Retrieve a list of active network names */
+    if (!VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_ACTIVE) ||
+        VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_ACTIVE)) {
+        if (virConnectListNetworks(ctl->conn,
+                                   names, nActiveNets) < 0) {
+            vshError(ctl, "%s", _("Failed to list active networks"));
+            goto cleanup;
+        }
+    }
+
+    /* Add the inactive networks to the end of the name list */
+    if (!VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_ACTIVE) ||
+        VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_ACTIVE)) {
+        if (virConnectListDefinedNetworks(ctl->conn,
+                                          &names[nActiveNets],
+                                          nInactiveNets) < 0) {
+            vshError(ctl, "%s", _("Failed to list inactive networks"));
+            goto cleanup;
+        }
+    }
+
+    list->nets = vshMalloc(ctl, sizeof(virNetworkPtr) * (nAllNets));
+    list->nnets = 0;
+
+    /* get active networks */
+    for (i = 0; i < nActiveNets; i++) {
+        if (!(net = virNetworkLookupByName(ctl->conn, names[i])))
+            continue;
+        list->nets[list->nnets++] = net;
+    }
+
+    /* get inactive networks */
+    for (i = 0; i < nInactiveNets; i++) {
+        if (!(net = virNetworkLookupByName(ctl->conn, names[i])))
+            continue;
+        list->nets[list->nnets++] = net;
+    }
+
+    /* truncate networks that weren't found */
+    deleted = nAllNets - list->nnets;
+
+filter:
+    /* filter list the list if the list was acquired by fallback means */
+    for (i = 0; i < list->nnets; i++) {
+        net = list->nets[i];
+
+        /* persistence filter */
+        if (VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_PERSISTENT)) {
+            if ((persistent = virNetworkIsPersistent(net)) < 0) {
+                vshError(ctl, "%s", _("Failed to get network persistence info"));
+                goto cleanup;
+            }
+
+            if (!((VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_PERSISTENT) && persistent) ||
+                  (VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_TRANSIENT) && !persistent)))
+                goto remove_entry;
+        }
+
+        /* autostart filter */
+        if (VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_FILTERS_AUTOSTART)) {
+            if (virNetworkGetAutostart(net, &autostart) < 0) {
+                vshError(ctl, "%s", _("Failed to get network autostart state"));
+                goto cleanup;
+            }
+
+            if (!((VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_AUTOSTART) && autostart) ||
+                  (VSH_MATCH(VIR_CONNECT_LIST_NETWORKS_NO_AUTOSTART) && !autostart)))
+                goto remove_entry;
+        }
+        /* the pool matched all filters, it may stay */
+        continue;
+
+remove_entry:
+        /* the pool has to be removed as it failed one of the filters */
+        virNetworkFree(list->nets[i]);
+        list->nets[i] = NULL;
+        deleted++;
+    }
+
+finished:
+    /* sort the list */
+    if (list->nets && list->nnets)
+        qsort(list->nets, list->nnets,
+              sizeof(*list->nets), vshNetworkSorter);
+
+    /* truncate the list if filter simulation deleted entries */
+    if (deleted)
+        VIR_SHRINK_N(list->nets, list->nnets, deleted);
+
+    success = true;
+
+cleanup:
+    for (i = 0; i < nAllNets; i++)
+        VIR_FREE(names[i]);
+    VIR_FREE(names);
+
+    if (!success) {
+        vshNetworkListFree(list);
+        list = NULL;
+    }
+
+    return list;
+}
+
 /*
  * "net-list" command
  */
@@ -372,117 +572,70 @@ static const vshCmdInfo info_network_list[] = {
 static const vshCmdOptDef opts_network_list[] = {
     {"inactive", VSH_OT_BOOL, 0, N_("list inactive networks")},
     {"all", VSH_OT_BOOL, 0, N_("list inactive & active networks")},
+    {"persistent", VSH_OT_BOOL, 0, N_("list persistent networks")},
+    {"transient", VSH_OT_BOOL, 0, N_("list transient networks")},
+    {"autostart", VSH_OT_BOOL, 0, N_("list networks with autostart enabled")},
+    {"no-autostart", VSH_OT_BOOL, 0, N_("list networks with autostart disabled")},
     {NULL, 0, 0, NULL}
 };
 
 static bool
 cmdNetworkList(vshControl *ctl, const vshCmd *cmd ATTRIBUTE_UNUSED)
 {
+    vshNetworkListPtr list = NULL;
+    int i;
     bool inactive = vshCommandOptBool(cmd, "inactive");
     bool all = vshCommandOptBool(cmd, "all");
-    bool active = !inactive || all;
-    int maxactive = 0, maxinactive = 0, i;
-    char **activeNames = NULL, **inactiveNames = NULL;
-    inactive |= all;
+    bool persistent = vshCommandOptBool(cmd, "persistent");
+    bool transient = vshCommandOptBool(cmd, "transient");
+    bool autostart = vshCommandOptBool(cmd, "autostart");
+    bool no_autostart = vshCommandOptBool(cmd, "no-autostart");
+    unsigned int flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
+    if (inactive)
+        flags = VIR_CONNECT_LIST_NETWORKS_INACTIVE;
+
+    if (all)
+        flags = VIR_CONNECT_LIST_NETWORKS_ACTIVE |
+                VIR_CONNECT_LIST_NETWORKS_INACTIVE;
+
+    if (persistent)
+         flags |= VIR_CONNECT_LIST_NETWORKS_PERSISTENT;
+
+    if (transient)
+         flags |= VIR_CONNECT_LIST_NETWORKS_TRANSIENT;
+
+    if (autostart)
+         flags |= VIR_CONNECT_LIST_NETWORKS_AUTOSTART;
+
+    if (no_autostart)
+         flags |= VIR_CONNECT_LIST_NETWORKS_NO_AUTOSTART;
+
+    if (!(list = vshNetworkListCollect(ctl, flags)))
         return false;
 
-    if (active) {
-        maxactive = virConnectNumOfNetworks(ctl->conn);
-        if (maxactive < 0) {
-            vshError(ctl, "%s", _("Failed to list active networks"));
-            return false;
-        }
-        if (maxactive) {
-            activeNames = vshMalloc(ctl, sizeof(char *) * maxactive);
+    vshPrintExtra(ctl, "%-20s %-10s %-13s %s\n", _("Name"), _("State"),
+                  _("Autostart"), _("Persistent"));
+    vshPrintExtra(ctl, "--------------------------------------------------\n");
 
-            if ((maxactive = virConnectListNetworks(ctl->conn, activeNames,
-                                                    maxactive)) < 0) {
-                vshError(ctl, "%s", _("Failed to list active networks"));
-                VIR_FREE(activeNames);
-                return false;
-            }
-
-            qsort(&activeNames[0], maxactive, sizeof(char *), vshNameSorter);
-        }
-    }
-    if (inactive) {
-        maxinactive = virConnectNumOfDefinedNetworks(ctl->conn);
-        if (maxinactive < 0) {
-            vshError(ctl, "%s", _("Failed to list inactive networks"));
-            VIR_FREE(activeNames);
-            return false;
-        }
-        if (maxinactive) {
-            inactiveNames = vshMalloc(ctl, sizeof(char *) * maxinactive);
-
-            if ((maxinactive =
-                     virConnectListDefinedNetworks(ctl->conn, inactiveNames,
-                                                   maxinactive)) < 0) {
-                vshError(ctl, "%s", _("Failed to list inactive networks"));
-                VIR_FREE(activeNames);
-                VIR_FREE(inactiveNames);
-                return false;
-            }
-
-            qsort(&inactiveNames[0], maxinactive, sizeof(char*), vshNameSorter);
-        }
-    }
-    vshPrintExtra(ctl, "%-20s %-10s %s\n", _("Name"), _("State"),
-                  _("Autostart"));
-    vshPrintExtra(ctl, "-----------------------------------------\n");
-
-    for (i = 0; i < maxactive; i++) {
-        virNetworkPtr network =
-            virNetworkLookupByName(ctl->conn, activeNames[i]);
+    for (i = 0; i < list->nnets; i++) {
+        virNetworkPtr network = list->nets[i];
         const char *autostartStr;
-        int autostart = 0;
+        int is_autostart = 0;
 
-        /* this kind of work with networks is not atomic operation */
-        if (!network) {
-            VIR_FREE(activeNames[i]);
-            continue;
-        }
-
-        if (virNetworkGetAutostart(network, &autostart) < 0)
+        if (virNetworkGetAutostart(network, &is_autostart) < 0)
             autostartStr = _("no autostart");
         else
-            autostartStr = autostart ? _("yes") : _("no");
+            autostartStr = is_autostart ? _("yes") : _("no");
 
-        vshPrint(ctl, "%-20s %-10s %-10s\n",
+        vshPrint(ctl, "%-20s %-10s %-13s %s\n",
                  virNetworkGetName(network),
-                 _("active"),
-                 autostartStr);
-        virNetworkFree(network);
-        VIR_FREE(activeNames[i]);
+                 virNetworkIsActive(network) ? _("active") : _("inactive"),
+                 autostartStr,
+                 virNetworkIsPersistent(network) ? _("yes") : _("no"));
     }
-    for (i = 0; i < maxinactive; i++) {
-        virNetworkPtr network = virNetworkLookupByName(ctl->conn, inactiveNames[i]);
-        const char *autostartStr;
-        int autostart = 0;
 
-        /* this kind of work with networks is not atomic operation */
-        if (!network) {
-            VIR_FREE(inactiveNames[i]);
-            continue;
-        }
-
-        if (virNetworkGetAutostart(network, &autostart) < 0)
-            autostartStr = _("no autostart");
-        else
-            autostartStr = autostart ? _("yes") : _("no");
-
-        vshPrint(ctl, "%-20s %-10s %-10s\n",
-                 inactiveNames[i],
-                 _("inactive"),
-                 autostartStr);
-
-        virNetworkFree(network);
-        VIR_FREE(inactiveNames[i]);
-    }
-    VIR_FREE(activeNames);
-    VIR_FREE(inactiveNames);
+    vshNetworkListFree(list);
     return true;
 }
 
@@ -505,8 +658,6 @@ cmdNetworkName(vshControl *ctl, const vshCmd *cmd)
 {
     virNetworkPtr network;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
     if (!(network = vshCommandOptNetworkBy(ctl, cmd, NULL,
                                            VSH_BYUUID)))
         return false;
@@ -537,9 +688,6 @@ cmdNetworkStart(vshControl *ctl, const vshCmd *cmd)
     bool ret = true;
     const char *name = NULL;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, &name)))
          return false;
 
@@ -574,9 +722,6 @@ cmdNetworkUndefine(vshControl *ctl, const vshCmd *cmd)
     bool ret = true;
     const char *name;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetwork(ctl, cmd, &name)))
         return false;
 
@@ -591,6 +736,160 @@ cmdNetworkUndefine(vshControl *ctl, const vshCmd *cmd)
     return ret;
 }
 
+/*
+ * "net-update" command
+ */
+static const vshCmdInfo info_network_update[] = {
+    {"help", N_("update parts of an existing network's configuration")},
+    {"desc", ""},
+    {NULL, NULL}
+};
+
+static const vshCmdOptDef opts_network_update[] = {
+    {"network", VSH_OT_DATA, VSH_OFLAG_REQ, N_("network name or uuid")},
+    {"command", VSH_OT_DATA, VSH_OFLAG_REQ,
+     N_("type of update (add-first, add-last (add), delete, or modify)")},
+    {"section", VSH_OT_DATA, VSH_OFLAG_REQ,
+     N_("which section of network configuration to update")},
+    {"xml", VSH_OT_DATA, VSH_OFLAG_REQ,
+     N_("name of file containing xml (or, if it starts with '<', the complete "
+        "xml element itself) to add/modify, or to be matched for search")},
+    {"parent-index", VSH_OT_INT, 0, N_("which parent object to search through")},
+    {"config", VSH_OT_BOOL, 0, N_("affect next boot")},
+    {"live", VSH_OT_BOOL, 0, N_("affect running domain")},
+    {"current", VSH_OT_BOOL, 0, N_("affect current domain")},
+    {NULL, 0, 0, NULL}
+};
+
+VIR_ENUM_DECL(virNetworkUpdateCommand)
+VIR_ENUM_IMPL(virNetworkUpdateCommand, VIR_NETWORK_UPDATE_COMMAND_LAST,
+              "none", "modify", "delete", "add-last", "add-first");
+
+VIR_ENUM_DECL(virNetworkSection)
+VIR_ENUM_IMPL(virNetworkSection, VIR_NETWORK_SECTION_LAST,
+              "none", "bridge", "domain", "ip", "ip-dhcp-host",
+              "ip-dhcp-range", "forward", "forward-interface",
+              "forward-pf", "portgroup", "dns-host", "dns-txt",
+              "dns-srv");
+
+static bool
+cmdNetworkUpdate(vshControl *ctl, const vshCmd *cmd)
+{
+    bool ret = false;
+    virNetworkPtr network;
+    const char *commandStr = NULL;
+    const char *sectionStr = NULL;
+    int command, section, parentIndex = -1;
+    const char *xml = NULL;
+    char *xmlFromFile = NULL;
+    bool current = vshCommandOptBool(cmd, "current");
+    bool config = vshCommandOptBool(cmd, "config");
+    bool live = vshCommandOptBool(cmd, "live");
+    unsigned int flags = 0;
+    const char *affected;
+
+    if (!(network = vshCommandOptNetwork(ctl, cmd, NULL)))
+        goto cleanup;
+
+    if (vshCommandOptString(cmd, "command", &commandStr) < 0) {
+        vshError(ctl, "%s", _("missing or malformed command argument"));
+        goto cleanup;
+    }
+
+    if (STREQ(commandStr, "add")) {
+        /* "add" is a synonym for "add-last" */
+        command = VIR_NETWORK_UPDATE_COMMAND_ADD_LAST;
+    } else {
+        command = virNetworkUpdateCommandTypeFromString(commandStr);
+        if (command <= 0 || command >= VIR_NETWORK_UPDATE_COMMAND_LAST) {
+            vshError(ctl, _("unrecognized command name '%s'"), commandStr);
+            goto cleanup;
+        }
+    }
+
+    if (vshCommandOptString(cmd, "section", &sectionStr) < 0) {
+        vshError(ctl, "%s", _("missing or malformed section argument"));
+        goto cleanup;
+    }
+    section = virNetworkSectionTypeFromString(sectionStr);
+    if (section <= 0 || section >= VIR_NETWORK_SECTION_LAST) {
+        vshError(ctl, _("unrecognized section name '%s'"), sectionStr);
+        goto cleanup;
+    }
+
+    if (vshCommandOptInt(cmd, "parent-index", &parentIndex) < 0) {
+        vshError(ctl, "%s", _("malformed parent-index argument"));
+        goto cleanup;
+    }
+
+    /* The goal is to have a full xml element in the "xml"
+     * string. This is provided in the --xml option, either directly
+     * (detected by the first character being "<"), or indirectly by
+     * supplying a filename (first character isn't "<") that contains
+     * the desired xml.
+     */
+
+    if (vshCommandOptString(cmd, "xml", &xml) < 0) {
+        vshError(ctl, "%s", _("malformed or missing xml argument"));
+        goto cleanup;
+    }
+
+    if (*xml != '<') {
+        /* contents of xmldata is actually the name of a file that
+         * contains the xml.
+         */
+        if (virFileReadAll(xml, VSH_MAX_XML_FILE, &xmlFromFile) < 0)
+            goto cleanup;
+        /* NB: the original xml is just a const char * that points
+         * to a string owned by the vshCmd object, and will be freed
+         * by vshCommandFree, so it's safe to lose its pointer here.
+         */
+        xml = xmlFromFile;
+    }
+
+    if (current) {
+        if (live || config) {
+            vshError(ctl, "%s", _("--current must be specified exclusively"));
+            return false;
+        }
+        flags |= VIR_NETWORK_UPDATE_AFFECT_CURRENT;
+    } else {
+        if (config)
+            flags |= VIR_NETWORK_UPDATE_AFFECT_CONFIG;
+        if (live)
+            flags |= VIR_NETWORK_UPDATE_AFFECT_LIVE;
+    }
+
+    if (virNetworkUpdate(network, command,
+                         section, parentIndex, xml, flags) < 0) {
+        vshError(ctl, _("Failed to update network %s"),
+                 virNetworkGetName(network));
+        goto cleanup;
+    }
+
+    if (config) {
+        if (live)
+            affected = _("persistent config and live state");
+        else
+            affected = _("persistent config");
+    } else if (live) {
+            affected = _("live state");
+    } else if (virNetworkIsActive(network)) {
+        affected = _("live state");
+    } else {
+        affected = _("persistent config");
+    }
+
+    vshPrint(ctl, _("Updated network %s %s"),
+             virNetworkGetName(network), affected);
+    ret = true;
+cleanup:
+    vshReportError(ctl);
+    virNetworkFree(network);
+    VIR_FREE(xmlFromFile);
+    return ret;
+}
+
 /*
  * "net-uuid" command
  */
@@ -611,9 +910,6 @@ cmdNetworkUuid(vshControl *ctl, const vshCmd *cmd)
     virNetworkPtr network;
     char uuid[VIR_UUID_STRING_BUFLEN];
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        return false;
-
     if (!(network = vshCommandOptNetworkBy(ctl, cmd, NULL,
                                            VSH_BYNAME)))
         return false;
@@ -664,9 +960,6 @@ cmdNetworkEdit(vshControl *ctl, const vshCmd *cmd)
     virNetworkPtr network = NULL;
     virNetworkPtr network_edited = NULL;
 
-    if (!vshConnectionUsability(ctl, ctl->conn))
-        goto cleanup;
-
     network = vshCommandOptNetwork(ctl, cmd, NULL);
     if (network == NULL)
         goto cleanup;
@@ -715,6 +1008,7 @@ const vshCmdDef networkCmds[] = {
     {"net-start", cmdNetworkStart, opts_network_start, info_network_start, 0},
     {"net-undefine", cmdNetworkUndefine, opts_network_undefine,
      info_network_undefine, 0},
+    {"net-update", cmdNetworkUpdate, opts_network_update, info_network_update, 0},
     {"net-uuid", cmdNetworkUuid, opts_network_uuid, info_network_uuid, 0},
     {NULL, NULL, NULL, NULL, 0}
 };
